<?php
/**
 * JetEngine Query Builder Add-On For Bricks Theme
 *
 * Plugin Name: JetEngine Query Builder Add-On For Bricks Theme
 * Plugin URI:  https://itchycode.com/integrate-jetengine-query-builder-in-bricks-query-loop-non-official/
 * Description: Non-official integration of JetEngine Query Builder with Bricks Theme.
 * Version:     1.0.0
 * Author:      Jenn Lee
 * Author URI:  https://itchycode.com/
 * License:     GPLv2 or later
 * Text Domain: itchy-bricks
 * Requires at least: 6.0
 * Requires PHP: 7.4
 *
 */

if (!defined('WPINC')) {
    die;
}

if (!class_exists('Itchy_Je_Qb_Bricks')) :

    define('ITCHY_JEQB_VERSION', '1.0.0');
    define('ITCHY_JEQB_PATH', plugin_dir_path(__FILE__));
    define('ITCHY_JEQB_URL', plugin_dir_url(__FILE__));

    class Itchy_Je_Qb_Bricks
    {

        public static function init_plugin()
        {

            $pass = self::check_requirement();

            if ($pass) {
                add_action('init', array(__CLASS__, 'itchy_add_control_to_elements'), 30);
                add_filter('bricks/setup/control_options', array(__CLASS__, 'itchy_setup_je_query_controls'));
                add_filter('bricks/query/run', array(__CLASS__, 'itchy_run_jeqb_query'), 10, 2);
                add_filter('bricks/query/loop_object', array(__CLASS__, 'itchy_set_jeqb_loop_object'), 10, 3);
                add_filter('bricks/dynamic_tags_list', array(__CLASS__, 'add_itchy_tag_to_builder'));
                add_filter('bricks/dynamic_data/render_content', array(__CLASS__, 'render_itchy_tag'), 10, 3);
                add_filter('bricks/frontend/render_data', array(__CLASS__, 'render_itchy_tag'), 10, 2);
                add_filter('bricks/dynamic_data/render_tag', array(__CLASS__, 'get_itchy_tag_value'), 10, 3);
            }

        }

        public static function check_requirement()
        {

            $theme = wp_get_theme();
            $active_theme_name = ($theme->parent()) ? $theme->parent()->get('Name') : $theme->get('Name');
            $active_theme_version = ($theme->parent()) ? $theme->parent()->get('Version') : $theme->get('Version');

            if ($active_theme_name !== 'Bricks') {
                add_action('admin_notices', array(__CLASS__, 'bricks_theme_required_notice'));
                return false;
            }

            if (version_compare($active_theme_version, '1.3.7', '<')) {
                add_action('admin_notices', array(__CLASS__, 'bricks_version_required_notice'));
                return false;
            }

            if (!class_exists('Jet_Engine')) {
                add_action('admin_notices', array(__CLASS__, 'jet_engine_required_notice'));
                return false;
            }

            return true;
        }

        public static function bricks_version_required_notice()
        {

            $class = 'notice notice-error';
            $message = __('JetEngine Query Builder Add-On For Bricks requires Bricks Theme higher than 1.3.7', 'itchy-bricks');
            printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));

        }

        public static function bricks_theme_required_notice()
        {

            $class = 'notice notice-error';
            $message = __('JetEngine Query Builder Add-On For Bricks requires Bricks Theme activated.', 'itchy-bricks');
            printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));

        }

        public static function jet_engine_required_notice()
        {

            $class = 'notice notice-error';
            $message = __('JetEngine Query Builder Add-On For Bricks requires JetEngine plugin activated.', 'itchy-bricks');
            printf('<div class="%1$s"><p>%2$s</p></div>', esc_attr($class), esc_html($message));

        }

        public static function itchy_setup_je_query_controls($control_options)
        {
            // Add a new query loop type
            $control_options['queryTypes']['je_qb'] = esc_html__('JE Query Builder', 'itchy-bricks');
            return $control_options;
        }

        public static function itchy_add_control_to_elements()
        {
            // Only container, block and div element have query controls
            $elements = ['container', 'block', 'div'];
            foreach ($elements as $name) {
                add_filter("bricks/elements/{$name}/controls", array(__CLASS__, 'itchy_add_je_controls'), 40);
            }
        }

        public static function itchy_add_je_controls($controls)
        {
            // je_qb_id will be my option key
            $je_control['je_qb_id'] = [
                'tab' => 'content',
                'label' => esc_html__('JE Queries', 'itchy-bricks'),
                'type' => 'select',
                'options' => [],
                'placeholder' => esc_html__('Choose a query', 'itchy-bricks'),
                'required' => array(
                    ['query.objectType', '=', 'je_qb'],
                    ['hasLoop', '!=', false]
                ),
                'rerender' => true,
                'description' => esc_html__('Please create a query in JetEngine Query Builder First', 'itchy-bricks'),
                'searchable' => true,
                'multiple' => false,
            ];

            // Get the defined queries from JetEngine Query Builder
            $je_queries = \Jet_Engine\Query_Builder\Manager::instance()->get_queries();
            // Add the queries to the options
            foreach ($je_queries as $id => $query) {
                $je_control['je_qb_id']['options'][$id] = $query->name;
            }

            // Below 2 lines is just some php array functions to force my new control located after the query control
            $query_key_index = absint(array_search('query', array_keys($controls)));
            $new_controls = array_slice($controls, 0, $query_key_index + 1, true) + $je_control + array_slice($controls, $query_key_index + 1, null, true);

            return $new_controls;
        }

        public static function itchy_run_jeqb_query($results, $query_obj)
        {
            // Only target if query type set is je_qb
            if ($query_obj->object_type !== 'je_qb') {
                return $results;
            }

            $settings = $query_obj->settings;
            $je_qb_id = absint($settings['je_qb_id']);

            // Return empty results if no query selected or Use Query is not checked
            if ($je_qb_id === 0 || !$settings['hasLoop']) {
                return [];
            }

            $query_builder = \Jet_Engine\Query_Builder\Manager::instance();

            // Get the query object from JetEngine based on the query id
            $je_query = $query_builder->get_query_by_id($je_qb_id);

            // Return empty results if query not found in JetEngine Query Builder
            if (!$je_query) {
                return [];
            }

            // Setup query args
            $je_query->setup_query();

            // Get the results
            return $je_query->get_items();
        }

        public static function itchy_set_jeqb_loop_object($loop_object, $loop_key, $query)
        {
            if ($query->object_type !== 'je_qb') {
                return $loop_object;
            }

            // $loop_object coming from the itchy_run_jeqb_query() function result
            // It could be in different format depending on the query type
            // Our main goal is to assign the correct post object to the $post global variable
            // So in the loop, when you use get_the_ID() or get_the_title() it will return the correct value

            global $post;

            // I only tested on JetEngine Posts Query, Terms Query, Comments Query and WC Products Query
            // I didn't set WP_Term condition because it's not related to the $post global variable
            if (is_a($loop_object, 'WP_Post')) {
                $post = $loop_object;
            } elseif (is_a($loop_object, 'WC_Product')) {
                // $post should be a WP_Post object
                $post = get_post($loop_object->get_id());
            } elseif (is_a($loop_object, 'WP_Comment')) {
                // A comment should refer to a post, so I set the $post global variable to the comment's post
                // You might want to change this to $loop_object->comment_ID
                $post = get_post($loop_object->comment_post_ID);
            }

            setup_postdata($post); // Honestly, this line might not be necessary

            // We still return the $loop_object so \Bricks\Query::get_loop_object() can use it
            return $loop_object;
        }

        public static function add_itchy_tag_to_builder($tags)
        {
            // Note that property is just a dummy tag, should be replaced with the actual property when using
            $tags[] = [
                'name' => '{itchy_extra:property}',
                'label' => 'Itchy Object Property',
                'group' => 'Itchy Extra Data'
            ];

            return $tags;
        }

        public static function render_itchy_tag($content, $post, $context = 'text')
        {

            // Only look for content starts with {itchy_extra:
            if (strpos($content, '{itchy_extra:') === false) {
                return $content;
            }

            // Regex to match itchy_extra: tag
            preg_match_all('/{(itchy_extra:[^}]+)}/', $content, $matches);

            // Nothing grouped in the regex, return the original content
            if (empty($matches[0])) {
                return $content;
            }

            // Get a list of tags to exclude from the Dynamic Data logic
            $exclude_tags = apply_filters('bricks/dynamic_data/exclude_tags', []);

            foreach ($matches[1] as $key => $match) {
                $tag = $matches[0][$key];

                if (in_array($match, $exclude_tags)) {
                    continue;
                }

                // Get the dynamic data value, $match is the tag name without the curly brackets
                $value = self::get_itchy_tag_value($match, $post, $context);

                // Replace the tag with the transformed value
                $content = str_replace($tag, $value, $content);
            }

            return $content;
        }

        public static function get_itchy_tag_value($tag, $post, $context = 'text')
        {

            // Only look for dynamic tag starts with itchy_extra:
            if (strpos($tag, 'itchy_extra:') === false) {
                return $tag;
            }

            // Get the property name
            $property = str_replace('itchy_extra:', '', $tag);

            // Use Bricks function to get the correct object
            if (\Bricks\Query::is_looping()) {
                $queried_object = \Bricks\Query::get_loop_object();
            } else {
                $queried_object = \Bricks\Helpers::get_queried_object($post->ID);
            }

            // If property is not found in the object, return a hint
            $value = $queried_object->{$property} ? $queried_object->{$property} : "Property '{$property}' not found in current object.";

            return $value;

        }


    }

    add_action('plugins_loaded', array('Itchy_Je_Qb_Bricks', 'init_plugin'));

endif;